<?php

/**
 * Description of cryptor_yform_autodelete
 *
 * @author Marshall
 */
class cryptor_yform_autodelete extends cryptor_yform {

    const CONFIG_ARRAY_KEY = 'autodelete_fields';
    const DEFAULT_DATESTAMP_PATTERN = 'Y-m-d H:i:s';

    private static $intervals = ['days' => 'DAY', 'weeks' => 'WEEK', 'months' => 'MONTH', 'years' => 'YEAR'];
    private static $fields = [];
    private static $tableConfig = null;
    private static $yFormTable = null;

    /**
     * Executes the autodelete on a table
     * @param <string> $tableName
     * @return <boolean>
     */
    public static function execute($tableName) {
        self::setTable($tableName);
        if (!self::isValidTable()) {
            return 'Not a valid autodelete table';
        }
        $deleteStatements = self::getDeleteStatements();
        if (count($deleteStatements)) {
            self::deleteTableEntriesWhere($tableName, implode(' OR ', $deleteStatements));
        }
    }

    /**
     * Sets the tableConfig, yFormTable and autodeleteFields by table name
     * @param <string> $tableName
     */
    public static function setTable($tableName) {
        self::$tableConfig = parent::getConfig($tableName);
        self::$yFormTable = rex_yform_manager_table::get($tableName);
        if (isset(self::$tableConfig[self::CONFIG_ARRAY_KEY])) {
            self::$fields = self::$tableConfig[self::CONFIG_ARRAY_KEY];
        }
    }

    /**
     * Returns true if given $format is a valid datetime format
     * @param <string> $format
     * @return <boolean>
     */
    public static function isValidDateFormat($format) {
        if ($format === '' || $format === 'mysql') {
            $format = self::DEFAULT_DATESTAMP_PATTERN;
        }
        return (\DateTime::createFromFormat($format, date($format)) !== false);
    }

    /**
     * Returns true if given $interval is a valid interval
     * @param <string> $interval
     * @return <boolean>
     */
    public static function isValidInterval($interval) {
        return in_array($interval, self::$intervals);
    }

    /**
     * Returns true if the table configuration seems to be valid
     * @return <boolean>
     */
    public static function isValidTable() {
        return (is_array(self::$tableConfig) && self::$yFormTable instanceof rex_yform_manager_table && is_array(self::$fields));
    }

    /**
     * Returns all supported auto delete intervals
     * @return <array> $autodeleteIntervals
     */
    public static function getIntervals() {
        return self::$intervals;
    }

    /**
     * Returns the format attribute of a yFormField
     * @param <object> $yFormField
     * @return <string> $format
     */
    public static function getDateFormat($yFormField) {
        $format = trim($yFormField->getElement('format'));
        if ($format === '' || $format === 'mysql') {
            return self::DEFAULT_DATESTAMP_PATTERN;
        }
        return $format;
    }

    /**
     * Returns the autodelete date, depending on the field settings or false if settings/value are invalid
     * @param <string> $tableName
     * @param <string> $fieldName
     * @param <string> $dateString
     * @return <mixed> $dateTimeObject | false
     */
    public static function getDeleteDate($tableName, $fieldName, $dateString = null) {
        // Get config for table and field
        $config = self::getConfig($tableName);
        $yFormTable = rex_yform_manager_table::get($tableName);
        if (!$yFormTable || !$config || !isset($config['autodelete_fields']) || !isset($config['autodelete_fields'][$fieldName])) {
            return false;
        }

        // Check if the field exists, has a number and an valid interval
        $field = $yFormTable->getValueField($fieldName);
        $number = (int) $config['autodelete_fields'][$fieldName]['number'];
        $interval = (string) $config['autodelete_fields'][$fieldName]['interval'];
        if (!$field || !$number || !self::isValidInterval($interval)) {
            return false;
        }

        // Get the formated value of this field
        $date = self::getDateTime($field, $dateString);
        if ($date === false) {
            return false;
        }

        // Finally calculate the autodelete date
        $date->modify('+' . $number . ' ' . strtolower($interval));
        return $date;
    }

    /**
     * Returns datetime object formated by the field attribute 'format'
     * @param <object> $yFormField
     * @param <string> $dateString
     * @return <mixed> $dateObject || false
     */
    private static function getDateTime($yFormField, $dateString) {
        $format = self::getDateFormat($yFormField);
        return \DateTime::createFromFormat($format, $dateString);
    }

    /**
     * Create the delete statements, based on given configuration
     * @return <array>
     */
    private static function getDeleteStatements() {
        $deleteStatements = [];
        foreach (self::$fields as $fieldName => $attributes) {

            // Attributes number and interval must be set
            $number = (int) $attributes['number'];
            $interval = (string) $attributes['interval'];
            if (!$number || !self::isValidInterval($interval)) {
                continue;
            }

            // yForm field must be exist and type of datestamp
            $yFormField = self::$yFormTable->getValueField($fieldName);
            if (!$yFormField || $yFormField->getTypeName() !== 'datestamp') {
                continue;
            }

            // yForm format must be a valid dateTime format
            $format = self::getDateFormat($yFormField);
            if (!$format || !self::isValidDateFormat($format)) {
                continue;
            }

            // Add the delete statement for this field
            $deleteStatements[] = '`' . $fieldName . '` < NOW() - INTERVAL ' . $number . ' ' . $interval;
        }
        return $deleteStatements;
    }

    /**
     * Delete table entries by where statement
     * @param <string> $tableName
     * @param <string> $where
     */
    private static function deleteTableEntriesWhere($tableName, $where) {
        $sql = rex_sql::factory();
//        $sql->setDebug(true);
        $query = 'DELETE FROM ' . $sql->escapeIdentifier($tableName) . ' WHERE ( ' . $where . ')';
        $sql->setQuery($query);
    }

}
